/***************************************************//**
 * @file    OBPReadSpectrumWithGainExchange.cpp
 * @date    March 2016
 * @author  Ocean Optics, Inc.
 *
 * LICENSE:
 *
 * SeaBreeze Copyright (C) 2016, Ocean Optics Inc
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject
 * to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *******************************************************/

#include "common/globals.h"
#include "vendors/OceanOptics/protocols/obp/exchanges/OBPReadSpectrumWithGainExchange.h"
#include "vendors/OceanOptics/protocols/obp/hints/OBPSpectrumHint.h"
#include "vendors/OceanOptics/protocols/obp/constants/OBPMessageTypes.h"
#include "vendors/OceanOptics/protocols/obp/exchanges/OBPMessage.h"
#include "common/UShortVector.h"
#include "common/DoubleVector.h"

using namespace seabreeze;
using namespace seabreeze::oceanBinaryProtocol;
using namespace std;

OBPReadSpectrumWithGainExchange::OBPReadSpectrumWithGainExchange(
        unsigned int readoutLength, unsigned int numPixels,
        GainAdjustedSpectrometerFeature *spec)
        : OBPReadSpectrumExchange(readoutLength, numPixels) {
    this->spectrometerFeature = spec;
}

OBPReadSpectrumWithGainExchange::~OBPReadSpectrumWithGainExchange() {

}

Data *OBPReadSpectrumWithGainExchange::transfer(TransferHelper *helper) {

    unsigned int i;
    Data *xfer;
    double maxIntensity;
    double saturationLevel;

    /* This will use the superclass to transfer data from the device, and will
     * then apply the intensity gain to the result.
     */
    xfer = OBPReadSpectrumExchange::transfer(helper);
    if(NULL == xfer) {
        string error("Expected Transfer::transfer to produce a non-null result "
                        "containing spectral data.  Without this data, it is not "
                        "possible to generate a valid formatted spectrum.");
        throw ProtocolException(error);
    }
    /* xfer should contain a UShortVector */

    if(NULL == this->spectrometerFeature) {
        /* FIXME: should this throw an illegal state exception instead? */
        return xfer;
    }

    maxIntensity = this->spectrometerFeature->getMaximumIntensity();
    saturationLevel = this->spectrometerFeature->getSaturationLevel();

    /* Cast the formatted values so that we can get to the array of shorts */
    UShortVector *usv = static_cast<UShortVector *>(xfer);
    vector<unsigned short> shortVec = usv->getUShortVector();

    /* Create a buffer to store the gain-adjusted values.  This is local. */
    vector<double> adjusted(this->numberOfPixels);

    for(i = 0; i < this->numberOfPixels; i++) {
        double temp = shortVec[i] * maxIntensity / saturationLevel;
        if(temp > maxIntensity) {
            temp = maxIntensity;
        }
        adjusted[i] = temp;
    }

    /* It might speed things up to dynamically allocate the buffer and
     * hand it off to retval rather than letting it make a copy.
     */
    DoubleVector *retval = new DoubleVector(adjusted);

    delete xfer;

    return retval;
}
